@@ -26,7 +26,7 @@ use crate::{
     session::Session,
 };
 
-use self::{builder::WithoutContainerId, engine::SessionId};
+use self::engine::SessionId;
 
 mod builder;
 pub use builder::*;
@@ -37,31 +37,44 @@ mod error;
 pub mod heartbeat;
 pub use error::*;
 
-/// Default max-frame-size
+/// Default max-frame-size.
+///
+/// Please note that this is different from `MaxFrameSize::default()`.
+/// This value is taken from `AmqpNetLite`
 pub const DEFAULT_MAX_FRAME_SIZE: u32 = 256 * 1024;
 
 /// Default channel-max
+///
+/// This value is taken from `AmqpNetLite`
 pub const DEFAULT_CHANNEL_MAX: u16 = 255;
 
 /// A handle to the [`Connection`] event loop.
 ///
-/// Dropping the handle will also stop the [`Connection`] event loop
-#[derive(Debug)]
-pub struct ConnectionHandle {
+/// Dropping the handle will also stop the [`Connection`] event loop.
+#[allow(dead_code)]
+pub struct ConnectionHandle<R> {
     pub(crate) control: Sender<ConnectionControl>,
-    handle: JoinHandle<Result<(), Error>>,
+    pub(crate) handle: JoinHandle<Result<(), Error>>,
 
     // outgoing channel for session
     pub(crate) outgoing: Sender<SessionFrame>,
+    pub(crate) session_listener: R,
 }
 
-impl Drop for ConnectionHandle {
+impl<R> std::fmt::Debug for ConnectionHandle<R> {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("ConnectionHandle")
+            .finish()
+    }
+}
+
+impl<R> Drop for ConnectionHandle<R> {
     fn drop(&mut self) {
         let _ = self.control.try_send(ConnectionControl::Close(None));
     }
 }
 
-impl ConnectionHandle {
+impl<R> ConnectionHandle<R> {
     /// Checks if the underlying event loop has stopped
     pub fn is_closed(&self) -> bool {
         self.control.is_closed()
@@ -168,15 +181,15 @@ impl ConnectionHandle {
 /// |`offered_capabilities`| `None` |
 /// |`desired_capabilities`| `None` |
 /// |`Properties`| `None` |
-/// 
+///
 /// # Order of negotiation
-/// 
+///
 /// The order of negotiation follows the priority below
-/// 
+///
 /// 1. TLS
 /// 2. SASL
 /// 3. AMQP
-/// 
+///
 /// # Customize configuration with [`Builder`]
 ///
 /// The example above creates a connection with the default configuration. If the user needs to customize the
@@ -194,20 +207,20 @@ impl ConnectionHandle {
 ///
 /// # TLS
 ///
-/// If "amqps" is found in url's scheme, the connection will start with exchanging TLS 
-/// protocol header (['A', 'M', 'Q', 'P', 1, 2, 0, 0]).
+/// If "amqps" is found in url's scheme, the connection will start with exchanging TLS
+/// protocol header (['A', 'M', 'Q', 'P', 2, 1, 0, 0]).
 /// TLS support is only enabled by selecting one and only one of the following feature flags
-/// 
+///
 /// 1. `"rustls"`: enables TLS support with `tokio-rustls`
 /// 2. `"native-tls"`: enables TLS support with `tokio-native-tls`
-/// 
+///
 /// ## Alternative Establishment
-/// 
-/// The specification allows establishing `Connection` on a pure TLS stream without exchanging the 
-/// TLS protocol header, and this can be accomplished using `Builder`'s `open_with_stream`. 
-/// An example of establishing connection on a `tokio_native_tls::TlsStream` is shown below. 
+///
+/// The specification allows establishing `Connection` on a pure TLS stream without exchanging the
+/// TLS protocol header, and this can be accomplished using `Builder`'s `open_with_stream`.
+/// An example of establishing connection on a `tokio_native_tls::TlsStream` is shown below.
 /// The `tls_stream` can be replaced with a `tokio_rustls::client::TlsStream`.
-/// 
+///
 /// ```rust,ignore
 /// let addr = "localhost:5671";
 /// let domain = "localhost";
@@ -215,7 +228,7 @@ impl ConnectionHandle {
 /// let connector = native_tls::TlsConnector::new();
 /// let connector = tokio_native_tls::TlsConnector::from(connector);
 /// let tls_stream = connector.connect(domain, stream).await.unwrap();
-/// 
+///
 /// let mut connection = Connection::builder()
 ///     .container_id("connection-1")
 ///     .scheme("amqp")
@@ -227,12 +240,12 @@ impl ConnectionHandle {
 ///     .await
 ///     .unwrap();
 /// ```
-/// 
+///
 /// ## TLS with feature `"rustls"` enabled
-/// 
+///
 /// TLS connection can be established with a default connector or a custom `tokio_rustls::TlsConnector`.
 /// The following connector is used unless a custom connector is supplied to the builder.
-/// 
+///
 /// ```rust,ignore
 /// let mut root_cert_store = RootCertStore::empty();
 /// root_cert_store.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.0.iter().map(
@@ -247,48 +260,48 @@ impl ConnectionHandle {
 /// let config = ClientConfig::builder()
 ///     .with_safe_defaults()
 ///     .with_root_certificates(root_cert_store)
-///     .with_no_client_auth(); 
+///     .with_no_client_auth();
 /// let connector = TlsConnector::from(Arc::new(config));
 /// ```
-/// 
+///
 /// Start TLS connection negotiation with default TLS connector
-/// 
+///
 /// ```rust,ignore
 /// let connector = Connection::open("amqps://guest:guest@localhost:5671").await.unwrap();
 /// ```
-/// 
+///
 /// Below shows how to use a custom `tokio_rustls::TlsConnector` for TLS.
-/// 
+///
 /// ```rust, ignore
 /// let config = rustls::ClientConfig::builder()
 ///     .with_safe_defaults()
 ///     .with_root_certificates(root_cert_store)
 ///     .with_no_client_auth(); // i guess this was previously the default?
 /// let connector = TlsConnector::from(Arc::new(config));
-/// 
+///
 /// let connection = Connection::builder()
 ///     .container_id("connection-1")
 ///     .tls_connector(connector)
 ///     .open("amqps://guest:guest@localhost:5671")
 ///     .await.unwrap();
 /// ```
-/// 
+///
 /// ## TLS with feature `"native-tls"` enabled
-/// 
+///
 /// TLS connection can be established with a default connector or a custom `tokio_native_tls::TlsConnector`.
 /// The following connector is used unless a custom connector is supplied to the builder.
-/// 
+///
 /// ```rust,ignore
 /// let connector = native_tls::TlsConnector::new().unwrap();
 /// let connector = tokio_native_tls::TlsConnector::from(connector);
 /// ```
-/// 
+///
 /// Below shows how to use a custom `tokio_native_tls::TlsConnector`.
-/// 
+///
 /// ```rust,ignore
-/// let connector = native_tls::TlsConnector::new();
+/// let connector = native_tls::TlsConnector::new().unwrap();
 /// let connector = tokio_native_tls::TlsConnector::from(connector);
-/// 
+///
 /// let connection = Connection::builder()
 ///     .container_id("connection-1")
 ///     .tls_connector(connector)
@@ -304,16 +317,16 @@ impl ConnectionHandle {
 /// found in the url will override whatever `SaslProfile` supplied to the [`Builder`].
 ///
 /// The examples below shows two ways of starting the connection with SASL negotiation.
-/// 
+///
 /// 1. Start SASL negotiation with SASL PLAIN profile extracted from the url
 ///
 ///     ```rust,ignore
 ///     let connection = Connection::open("connection-1", "amqp://guest:guest@localhost:5672").await.unwrap();
 ///     ```
-/// 
-/// 2. Start SASL negotiation with the builder. Please note that tf the url contains `username` and `password`, 
+///
+/// 2. Start SASL negotiation with the builder. Please note that tf the url contains `username` and `password`,
 /// the profile supplied to the builder will be overriden.
-/// 
+///
 ///     ```
 ///     // This is equivalent to the line above
 ///     let profile = SaslProfile::Plain {
@@ -329,26 +342,26 @@ impl ConnectionHandle {
 ///
 #[derive(Debug)]
 pub struct Connection {
-    control: Sender<ConnectionControl>,
+    pub(crate) control: Sender<ConnectionControl>,
 
     // local
-    local_state: ConnectionState,
-    local_open: Open,
-    local_sessions: Slab<Sender<SessionIncomingItem>>,
-    session_by_incoming_channel: BTreeMap<u16, usize>,
-    session_by_outgoing_channel: BTreeMap<u16, usize>,
+    pub(crate) local_state: ConnectionState,
+    pub(crate) local_open: Open,
+    pub(crate) local_sessions: Slab<Sender<SessionIncomingItem>>,
+    pub(crate) session_by_incoming_channel: BTreeMap<u16, usize>,
+    pub(crate) session_by_outgoing_channel: BTreeMap<u16, usize>,
 
     // remote
-    remote_open: Option<Open>,
+    pub(crate) remote_open: Option<Open>,
 
     // mutually agreed channel max
-    agreed_channel_max: u16,
+    pub(crate) agreed_channel_max: u16,
 }
 
 /* ------------------------------- Public API ------------------------------- */
 impl Connection {
     /// Creates a Builder for [`Connection`]
-    pub fn builder<'a>() -> builder::Builder<'a, WithoutContainerId, ()> {
+    pub fn builder<'a>() -> builder::Builder<'a, mode::ConnectorNoId, ()> {
         builder::Builder::new()
     }
 
@@ -374,22 +387,22 @@ impl Connection {
     /// ```rust, ignore
     /// let connection = Connection::open("connection-1", "amqp://localhost:5672").await.unwrap();
     /// ```
-    /// 
+    ///
     /// # TLS
-    /// 
+    ///
     /// TLS support is enabled by selecting one and only one of the following feature flags
-    /// 
+    ///
     /// 1. `"rustls"`: enables TLS support with `tokio-rustls`
     /// 2. `"native-tls"`: enables TLS support with `tokio-native-tls`
-    /// 
+    ///
     /// ```rust, ignore
     /// let connection = Connection::open("connection-1", "amqps://localhost:5671").await.unwrap();
     /// ```
     /// ## TLS with feature `"rustls"` enabled
-    /// 
+    ///
     /// TLS connection can be established with a default connector or a custom `tokio_rustls::TlsConnector`.
     /// The following connector is used unless a custom connector is supplied to the builder.
-    /// 
+    ///
     /// ```rust,ignore
     /// let mut root_cert_store = RootCertStore::empty();
     /// root_cert_store.add_server_trust_anchors(webpki_roots::TLS_SERVER_ROOTS.0.iter().map(
@@ -404,24 +417,24 @@ impl Connection {
     /// let config = ClientConfig::builder()
     ///     .with_safe_defaults()
     ///     .with_root_certificates(root_cert_store)
-    ///     .with_no_client_auth(); 
+    ///     .with_no_client_auth();
     /// let connector = TlsConnector::from(Arc::new(config));
     /// ```
-    /// 
+    ///
     /// ### TLS with feature `"native-tls"` enabled
-    /// 
+    ///
     /// TLS connection can be established with a default connector or a custom `tokio_native_tls::TlsConnector`.
     /// The following connector is used unless a custom connector is supplied to the builder.
-    /// 
+    ///
     /// ```rust,ignore
     /// let connector = native_tls::TlsConnector::new().unwrap();
     /// let connector = tokio_native_tls::TlsConnector::from(connector);
     /// ```
-    /// 
+    ///
     /// # SASL
     ///
     /// Start SASL negotiation with SASL PLAIN profile extracted from the url
-    /// 
+    ///
     /// ```rust, ignore
     /// let connection = Connection::open("connection-1", "amqp://guest:guest@localhost:5672").await.unwrap();
     /// ```
@@ -429,7 +442,7 @@ impl Connection {
     pub async fn open(
         container_id: impl Into<String>, // TODO: default container id? random uuid-ish
         url: impl TryInto<Url, Error = url::ParseError>,
-    ) -> Result<ConnectionHandle, OpenError> {
+    ) -> Result<ConnectionHandle<()>, OpenError> {
         Connection::builder()
             .container_id(container_id)
             .open(url)
@@ -439,7 +452,7 @@ impl Connection {
 
 /* ------------------------------- Private API ------------------------------ */
 impl Connection {
-    fn new(
+    pub(crate) fn new(
         control: Sender<ConnectionControl>,
         local_state: ConnectionState,
         local_open: Open,
@@ -514,7 +527,7 @@ impl endpoint::Connection for Connection {
     }
 
     /// Reacting to remote Open frame
-    #[instrument(name = "RECV", skip_all)]
+    #[instrument(skip_all)]
     async fn on_incoming_open(&mut self, channel: u16, open: Open) -> Result<(), Self::Error> {
         trace!(channel, frame = ?open);
         match &self.local_state {
@@ -532,35 +545,14 @@ impl endpoint::Connection for Connection {
     }
 
     /// Reacting to remote Begin frame
-    #[instrument(name = "RECV", skip_all)]
+    #[instrument(skip_all)]
     async fn on_incoming_begin(&mut self, channel: u16, begin: Begin) -> Result<(), Self::Error> {
         trace!(channel, frame = ?begin);
-        match &self.local_state {
-            ConnectionState::Opened => {}
-            // TODO: what about pipelined
-            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)), // TODO: what to do?
-        }
-
-        match begin.remote_channel {
-            Some(outgoing_channel) => {
-                let session_id = self
-                    .session_by_outgoing_channel
-                    .get(&outgoing_channel)
-                    .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?; // Close with error NotFound
-
-                if self.session_by_incoming_channel.contains_key(&channel) {
-                    return Err(Error::amqp_error(AmqpError::NotAllowed, None)); // TODO: this is probably not how not allowed should be used?
-                }
-                self.session_by_incoming_channel
-                    .insert(channel, *session_id);
-
+        match self.on_incoming_begin_inner(channel, &begin)? {
+            Some(session_id) => {
                 // forward begin to session
-                let tx = self
-                    .local_sessions
-                    .get_mut(*session_id)
-                    .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?;
                 let sframe = SessionFrame::new(channel, SessionFrameBody::Begin(begin));
-                tx.send(sframe).await?;
+                self.send_to_session(session_id, sframe).await?;
             }
             None => {
                 // If a session is locally initiated, the remote-channel MUST NOT be set. When an endpoint responds
@@ -578,7 +570,7 @@ impl endpoint::Connection for Connection {
     }
 
     /// Reacting to remote End frame
-    #[instrument(name = "RECV", skip_all)]
+    #[instrument(skip_all)]
     async fn on_incoming_end(&mut self, channel: u16, end: End) -> Result<(), Self::Error> {
         trace!(channel, frame = ?end);
         match &self.local_state {
@@ -603,7 +595,7 @@ impl endpoint::Connection for Connection {
     }
 
     /// Reacting to remote Close frame
-    #[instrument(name = "RECV", skip_all)]
+    #[instrument(skip_all)]
     async fn on_incoming_close(&mut self, channel: u16, close: Close) -> Result<(), Self::Error> {
         trace!(channel, frame=?close);
 
@@ -644,14 +636,13 @@ impl endpoint::Connection for Connection {
         Ok(())
     }
 
-    fn on_outgoing_begin(&mut self, channel: u16, begin: Begin) -> Result<Frame, Self::Error> {
-        // TODO: the engine already checks that
-        // match &self.local_state {
-        //     ConnectionState::Opened => {}
-        //     _ => return Err(Error::Message("Illegal local connection state")),
-        // }
-
-        let frame = Frame::new(channel, FrameBody::Begin(begin));
+    fn on_outgoing_begin(
+        &mut self,
+        outgoing_channel: u16,
+        begin: Begin,
+    ) -> Result<Frame, Self::Error> {
+        // TODO: check states?
+        let frame = Frame::new(outgoing_channel, FrameBody::Begin(begin));
         Ok(frame)
     }
 
@@ -705,3 +696,59 @@ impl endpoint::Connection for Connection {
         self.local_sessions.get_mut(*session_id)
     }
 }
+
+impl Connection {
+    pub(crate) async fn send_to_session(
+        &mut self,
+        session_id: usize,
+        frame: SessionFrame,
+    ) -> Result<(), Error> {
+        let tx = self
+            .local_sessions
+            .get_mut(session_id)
+            .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?;
+        tx.send(frame).await?;
+        Ok(())
+    }
+
+    pub(crate) fn on_incoming_begin_inner(
+        &mut self,
+        channel: u16,
+        begin: &Begin,
+    ) -> Result<Option<usize>, Error> {
+        match &self.local_state {
+            ConnectionState::Opened => {}
+            // TODO: what about pipelined
+            _ => return Err(Error::amqp_error(AmqpError::IllegalState, None)), // TODO: what to do?
+        }
+
+        match begin.remote_channel {
+            // This corresponds a locally initiated session
+            Some(outgoing_channel) => {
+                let session_id = self
+                    .session_by_outgoing_channel
+                    .get(&outgoing_channel)
+                    .ok_or_else(|| Error::amqp_error(AmqpError::NotFound, None))?; // Close with error NotFound
+
+                if self.session_by_incoming_channel.contains_key(&channel) {
+                    return Err(Error::amqp_error(AmqpError::NotAllowed, None)); // TODO: this is probably not how not allowed should be used?
+                }
+                self.session_by_incoming_channel
+                    .insert(channel, *session_id);
+                Ok(Some(*session_id))
+            }
+            // This corresponds to remotely initated session
+            None => {
+                // If a session is locally initiated, the remote-channel MUST NOT be set. When an endpoint responds
+                // to a remotely initiated session, the remote-channel MUST be set to the channel on which the
+                // remote session sent the begin.
+                // TODO: allow remotely initiated session
+                // return Err(Error::amqp_error(
+                //     AmqpError::NotImplemented,
+                //     Some("Remotely initiazted session is not supported yet".to_string()),
+                // )); // Close with error NotImplemented
+                Ok(None)
+            }
+        }
+    }
+}
