@@ -2,10 +2,13 @@ use fe2o3_amqp_types::performatives::{Attach, Detach, Disposition, Transfer};
 
 use crate::{endpoint::LinkFlow, Payload};
 
-pub type LinkIncomingItem = LinkFrame;
+pub(crate) type LinkIncomingItem = LinkFrame;
 
+/// Link frames.
+///
+/// This is a subset of the AMPQ frames
 #[derive(Debug)]
-pub enum LinkFrame {
+pub(crate) enum LinkFrame {
     Attach(Attach),
     Flow(LinkFlow),
     Transfer {
