@@ -3,28 +3,26 @@
 use std::collections::BTreeMap;
 
 use fe2o3_amqp_types::definitions::{Fields, Handle, TransferNumber};
-use serde_amqp::primitives::{Symbol, UInt};
+use serde_amqp::primitives::Symbol;
 use slab::Slab;
 use tokio::sync::mpsc;
 use tokio_util::sync::PollSender;
 
 use crate::{
-    connection::{ConnectionHandle, DEFAULT_OUTGOING_BUFFER_SIZE},
+    connection::ConnectionHandle,
     control::SessionControl,
     session::{engine::SessionEngine, SessionState},
     util::Constant,
+    Session,
 };
 
-use super::{Error, SessionHandle};
+use super::{Error, SessionHandle, DEFAULT_WINDOW};
 
 pub(crate) const DEFAULT_SESSION_CONTROL_BUFFER_SIZE: usize = 128;
 pub(crate) const DEFAULT_SESSION_MUX_BUFFER_SIZE: usize = u16::MAX as usize;
 
-/// Default incoming_window and outgoing_window
-pub const DEFAULT_WINDOW: UInt = 2048;
-
 /// Builder for [`crate::Session`]
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Builder {
     /// The transfer-id of the first transfer id the sender will send
     pub next_outgoing_id: TransferNumber,
@@ -52,9 +50,8 @@ pub struct Builder {
     pub buffer_size: usize,
 }
 
-impl Builder {
-    /// Creates a new builder for [`crate::Session`]
-    pub fn new() -> Self {
+impl Default for Builder {
+    fn default() -> Self {
         Self {
             next_outgoing_id: 0,
             incoming_window: DEFAULT_WINDOW,
@@ -66,6 +63,44 @@ impl Builder {
             buffer_size: DEFAULT_SESSION_MUX_BUFFER_SIZE,
         }
     }
+}
+
+impl Builder {
+    /// Creates a new builder for [`crate::Session`]
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    pub(crate) fn into_session(
+        self,
+        control: mpsc::Sender<SessionControl>,
+        outgoing_channel: u16,
+        local_state: SessionState,
+    ) -> Session {
+        Session {
+            control,
+            // session_id,
+            outgoing_channel,
+            local_state,
+            initial_outgoing_id: Constant::new(self.next_outgoing_id),
+            next_outgoing_id: self.next_outgoing_id,
+            incoming_window: self.incoming_window,
+            outgoing_window: self.outgoing_window,
+            handle_max: self.handle_max,
+            incoming_channel: None,
+            next_incoming_id: 0,
+            remote_incoming_window: 0,
+            remote_outgoing_window: 0,
+            offered_capabilities: self.offered_capabilities,
+            desired_capabilities: self.desired_capabilities,
+            properties: self.properties,
+
+            local_links: Slab::new(),
+            link_by_name: BTreeMap::new(),
+            link_by_input_handle: BTreeMap::new(),
+            delivery_tag_by_id: BTreeMap::new(),
+        }
+    }
 
     /// The transfer-id of the first transfer id the sender will send
     pub fn next_outgoing_id(mut self, value: TransferNumber) -> Self {
@@ -144,49 +179,28 @@ impl Builder {
     ///     .begin(&mut connection)
     ///     .await.unwrap();
     /// ```
-    /// 
-    pub async fn begin(self, conn: &mut ConnectionHandle) -> Result<SessionHandle, Error> {
-        use super::Session;
-
+    ///
+    pub async fn begin(
+        self,
+        connection: &mut ConnectionHandle<()>,
+    ) -> Result<SessionHandle<()>, Error> {
         let local_state = SessionState::Unmapped;
         let (session_control_tx, session_control_rx) =
             mpsc::channel::<SessionControl>(DEFAULT_SESSION_CONTROL_BUFFER_SIZE);
         let (incoming_tx, incoming_rx) = mpsc::channel(self.buffer_size);
-        let (outgoing_tx, outgoing_rx) = mpsc::channel(DEFAULT_OUTGOING_BUFFER_SIZE);
+        let (outgoing_tx, outgoing_rx) = mpsc::channel(self.buffer_size);
 
         // create session in connection::Engine
-        let (outgoing_channel, session_id) = conn.allocate_session(incoming_tx).await?; // AllocSessionError
+        let (outgoing_channel, session_id) = connection.allocate_session(incoming_tx).await?; // AllocSessionError
 
-        let session = Session {
-            control: session_control_tx.clone(),
-            // session_id,
-            outgoing_channel,
-            local_state,
-            initial_outgoing_id: Constant::new(self.next_outgoing_id),
-            next_outgoing_id: self.next_outgoing_id,
-            incoming_window: self.incoming_window,
-            outgoing_window: self.outgoing_window,
-            handle_max: self.handle_max,
-            incoming_channel: None,
-            next_incoming_id: 0,
-            remote_incoming_window: 0,
-            remote_outgoing_window: 0,
-            offered_capabilities: self.offered_capabilities,
-            desired_capabilities: self.desired_capabilities,
-            properties: self.properties,
-
-            local_links: Slab::new(),
-            link_by_name: BTreeMap::new(),
-            link_by_input_handle: BTreeMap::new(),
-            delivery_tag_by_id: BTreeMap::new(),
-        };
-        let engine = SessionEngine::begin(
-            conn.control.clone(),
+        let session = self.into_session(session_control_tx.clone(), outgoing_channel, local_state);
+        let engine = SessionEngine::<crate::Session>::begin(
+            connection.control.clone(),
             session,
             session_id,
             session_control_rx,
             incoming_rx,
-            PollSender::new(conn.outgoing.clone()),
+            PollSender::new(connection.outgoing.clone()),
             outgoing_rx,
         )
         .await?;
@@ -196,6 +210,7 @@ impl Builder {
             control: session_control_tx,
             engine_handle,
             outgoing: outgoing_tx,
+            link_listener: (),
         };
         Ok(handle)
     }
